// ParticlePanel class
// CSC 375 - Lea
// Jacob Peck - 20110913
// Assignment 1 - Threaded Particles

import java.util.*;
import java.util.concurrent.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ParticlePanel extends JPanel {
  // hold the particles and threads
  private final CopyOnWriteArrayList<Particle> particles = new CopyOnWriteArrayList<Particle>();
  private CopyOnWriteArrayList<Thread> threads;
  
  // configurable parameters
  private static final int initialParticleCount = 5; // particles to start
  private static final int sleepInterval = 50;  // millisecond sleep between individual ticks
  

  // ctor, sets up properties and adds a simple mouse click event listener
  public ParticlePanel() {
    super();
    super.setBackground(Color.white);
    super.setOpaque(true);
    addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        addParticle(e.getX(),e.getY());
      }
    });
  }
  
  // utility method, stolen shamelessly from DL's example code
  // Returns a new Thread, consisting of a move/draw/sleep loop.
  private Thread makeThread(final Particle p) {
    Runnable runloop = new Runnable() {
      public void run() {
        try {
          for(;;) {
            p.move(getWidth(), getHeight(),particles);
            repaint();
            Thread.sleep(sleepInterval);
          }
        }
        catch (InterruptedException e) { return; }
      }
    };
    return new Thread(runloop);
  }
  
  // start the system and make all initial particles and threads
  public void go() {
    if (threads == null) { // bypass if already started
      initializeParticles(initialParticleCount);

      threads = new CopyOnWriteArrayList<Thread>();
      for (int i = 0; i < initialParticleCount; ++i) {
        threads.add(makeThread(particles.get(i)));
        threads.get(i).start();
      }
    }
  }
  
  
  // draw all the particles
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    for (Particle p : particles) {
      p.draw(g);
    }
  }

  // create the initial particles
  // started in random initial positions with random velocities, random sizes,
  // and random colors.
  private void initializeParticles(int n) {
    int width = getWidth();
    int height = getHeight();
    Random random = new Random();
    for (int i = 0; i < n; i++) {
      particles.add(new Particle(random.nextInt(width), random.nextInt(height),
                    -20 + random.nextInt(40), -20 + random.nextInt(40), 
                    new Color(random.nextInt(400000000)),
                    random.nextInt(40) + 5));
    }
  }
  
  
  // for mouse click event, creates a new particle and accompanying thread,
  // and sets it in motion by starting the thread
  private void addParticle(int x, int y) {
    Random random = new Random();
    synchronized(this) {
      particles.add(new Particle(x, y, -20 + random.nextInt(40), -20 + random.nextInt(40), 
                    new Color(random.nextInt(400000000)), random.nextInt(40) + 5));
      threads.add(makeThread(particles.get(particles.size()-1)));
      threads.get(threads.size()-1).start();
    }
  }
}
